// Developed by Paden Hogeland
//   Uses code from Rajarshi Roy http://siliconrepublic.blogspot.com/2011/02/arduino-based-pc-ambient-lighting.html
// 
//   This program will take screen shots, compute the average RGB of the left, top left, top right, right sides of the screen shot
//     and send this information to the arduino
//
//   Screen Resolution: 1440x900

// Import Libraries
import java.awt.Robot; // Use to take screenshots
import java.awt.AWTException;
import java.awt.event.InputEvent;
import java.awt.image.BufferedImage;
import java.awt.Rectangle;
import java.awt.Dimension;
import processing.serial.*; // Used for serial communication


Serial port; //creates object "port" of serial class
Robot robby; //creates object "robby" of robot class

// Set up Loop
void setup(){
  // Set up the serial connection Note: May need to change serial port number
  port = new Serial(this, Serial.list()[1],9600);
  // Initalize the window size
  size(100, 100);
  // Try to create robot 
  try{
    robby = new Robot();
  }
  // If an error
  catch (AWTException e){
    println("Robot class not supported by your system!");
    exit();
  }
}

// Draw Loop
void draw(){
  
  // Declare Variables hold the 32 bite result - ARGB(Alpha,Red,Green,Blue) - 8 bites each
  int pixel;
  // Initalize RGB variables
  float redLeft=0;
  float greenLeft=0;
  float blueLeft=0;
  float redRight=0;
  float greenRight = 0;
  float blueRight = 0;
  float redTopLeft = 0;  
  float greenTopLeft = 0;
  float blueTopLeft = 0;
  float redTopRight = 0;
  float greenTopRight = 0;
  float blueTopRight = 0;
  float r,g,b;

  // Get screenshot 
  BufferedImage screenshot = robby.createScreenCapture(new Rectangle(new Dimension(1440,900)));
  int i,j;
  // Calcualte the Left Side RGB Values
  // For 360 pixles accross (Width)
  for(i =0;i<360; i=i+2){
    // For 900 pixles down (Height)
    for(j=0; j<900;j=j+2){
      // Get pixel value - returns 32 bytes 
      pixel = screenshot.getRGB(i,j); 
      // Position to red bytes and add up
      redLeft = redLeft+(int)(255&(pixel>>16));
      // Position to green bytes and add up
      greenLeft = greenLeft+(int)(255&(pixel>>8)); 
      // Position to blue bytes and add up
      blueLeft = blueLeft+(int)(255&(pixel));
    }
  }

  // Average the Colors (Remember we only added every other pixel)
  r = redLeft/(180*450); 
  g = greenLeft/(180*450); 
  b = blueLeft/(180*450); 

if(r <= 25){
  r=0;
}
if(g<=25){
  g=0;
}
if(b<=25){
  b=0;
}
port.write(0xff); //write marker (0xff) for synchronization
port.write((byte)(r)); //write red value
port.write((byte)(g)); //write green value
port.write((byte)(b)); //write blue value

// Width x Height
// 1440*900
// Calculate the Top Left RGB Values
for(i =0;i<720; i=i+2){
  for(j=0; j<90;j=j+2){
    pixel = screenshot.getRGB(i,j); //the ARGB integer has the colors of pixel (i,j)
    redTopLeft = redTopLeft+(int)(255&(pixel>>16)); //add up reds
    greenTopLeft = greenTopLeft+(int)(255&(pixel>>8)); //add up greens
    blueTopLeft = blueTopLeft+(int)(255&(pixel)); //add up blues
  }
}

r=redTopLeft/(360*45); //average red (remember that I skipped ever alternate pixel)
g=greenTopLeft/(360*45); //average green
b=blueTopLeft/(360*45); //average blue
if(r <= 25){
  r=0;
}
if(g<=25){
  g=0;
}
if(b<=25){
  b=0;
}
port.write((byte)(r)); //write red value
port.write((byte)(g)); //write green value
port.write((byte)(b)); //write blue value


// Width x Height
// 1440*900
// Calculate the Top Right RGB Values
for(i =720;i<1440; i=i+2){
  for(j=0; j<90;j=j+2){
    pixel = screenshot.getRGB(i,j); //the ARGB integer has the colors of pixel (i,j)
    redTopRight = redTopRight+(int)(255&(pixel>>16)); //add up reds
    greenTopRight = greenTopRight+(int)(255&(pixel>>8)); //add up greens
    blueTopRight = blueTopRight+(int)(255&(pixel)); //add up blues
  }
}

r=redTopRight/(360*45); //average red (remember that I skipped ever alternate pixel)
g=greenTopRight/(360*45); //average green
b=blueTopRight/(360*45); //average blue
if(r <= 25){
  r=0;
}
if(g<=25){
  g=0;
}
if(b<=25){
  b=0;
}
port.write((byte)(r)); //write red value
port.write((byte)(g)); //write green value
port.write((byte)(b)); //write blue value

// Width x Height
// 1368*928
//I skip every alternate pixel making my program 4 times faster
for(i =1080; i<1440; i=i+2){
  for(j=0; j<900;j=j+2){
    pixel = screenshot.getRGB(i,j); //the ARGB integer has the colors of pixel (i,j)
    redRight = redRight+(int)(255&(pixel>>16)); //add up reds
    greenRight = greenRight+(int)(255&(pixel>>8)); //add up greens
    blueRight = blueRight+(int)(255&(pixel)); //add up blues
  }
}

r=redRight/(180*450); //average red (remember that I skipped ever alternate pixel)
g=greenRight/(180*450); //average green
b=blueRight/(180*450); //average blue
if(r <= 25){
  r=0;
}
if(g<=25){
  g=0;
}
if(b<=25){
  b=0;
}
port.write((byte)(r)); //write red value
port.write((byte)(g)); //write green value
port.write((byte)(b)); //write blue value

delay(10); //delay for safety

background(r,g,b); //make window background average color
}
